const express = require('express');
const multer = require('multer');
const crypto = require('crypto');
const fs = require('fs-extra');
const bodyParser = require('body-parser');
const md5 = require('md5');
const path = require('path');

const app = express();
const port = 4444;
const uploadDir = './uploads/';
const filesMetadataPath = './files.json';
const logsPath = './logs.txt';

app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: true }));
app.use(express.static(uploadDir));
app.use(express.static('public')); // Serve static files from the public directory

fs.ensureDirSync(uploadDir);
fs.ensureFileSync(filesMetadataPath);
fs.ensureFileSync(logsPath);

let filesMetadata = JSON.parse(fs.readFileSync(filesMetadataPath, 'utf8') || '{}');

// Configure multer for file uploads
const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, uploadDir);
  },
  filename: function (req, file, cb) {
    crypto.randomBytes(16, (err, buf) => {
      if (err) return cb(err);
      cb(null, buf.toString('hex') + path.extname(file.originalname));
    });
  }
});

const upload = multer({
  storage: storage,
  limits: { fileSize: 100 * 1024 * 1024 }, // 100MB limit
}).single('file');

// Helper function to get the full URL
function getFullUrl(req) {
  const protocol = req.headers['x-forwarded-proto'] || req.protocol;
  return `${protocol}://${req.get('host')}`;
}

// Upload endpoint
app.post('/upload', (req, res) => {
  upload(req, res, (err) => {
    if (err) {
      return res.status(500).json({ error: err.message });
    }

    const { originalname, filename, path: filePath, size } = req.file;
    const { downloads = 1, password = null, expiry = 180 } = req.body;

    const md5Hash = md5(fs.readFileSync(filePath));
    const fileId = crypto.randomBytes(8).toString('hex');
    const expiryTime = Math.min(Math.max(parseInt(expiry, 10), 1), 180) * 60 * 1000; // Ensure expiry is between 1 minute and 3 hours

    filesMetadata[fileId] = {
      originalname,
      filename,
      path: filePath,
      size,
      downloads: parseInt(downloads, 10),
      password,
      expiry: Date.now() + expiryTime,
      md5: md5Hash,
      uploadTime: Date.now()
    };

    fs.writeFileSync(filesMetadataPath, JSON.stringify(filesMetadata, null, 2));
    fs.appendFileSync(logsPath, `${req.ip}, ${originalname}, ${md5Hash}\n`);

    res.json({ url: `${getFullUrl(req)}/${fileId}/${originalname}` });
  });
});

// Download endpoint
app.get('/:fileId/:filename', (req, res) => {
  const { fileId, filename } = req.params;
  const fileMetadata = filesMetadata[fileId];

  if (!fileMetadata) {
    return res.status(404).json({ error: 'File not found' });
  }

  if (fileMetadata.expiry < Date.now()) {
    return res.status(410).json({ error: 'File has expired' });
  }

  if (fileMetadata.downloads <= 0) {
    return res.status(410).json({ error: 'Download limit reached' });
  }

  const providedPassword = req.query.password || req.get('Authorization');

  if (fileMetadata.password && fileMetadata.password !== providedPassword) {
    return res.status(403).send(`
      <html>
        <body>
          <form action="" method="get">
            <label for="password">Password:</label>
            <input type="password" id="password" name="password" required>
            <button type="submit">Submit</button>
          </form>
        </body>
      </html>
    `);
  }

  fileMetadata.downloads -= 1;
  fs.writeFileSync(filesMetadataPath, JSON.stringify(filesMetadata, null, 2));

  res.download(fileMetadata.path, fileMetadata.originalname);
});

// Cleanup expired files
setInterval(() => {
  const now = Date.now();
  for (const [fileId, metadata] of Object.entries(filesMetadata)) {
    if (metadata.expiry < now || metadata.downloads <= 0) {
      fs.remove(metadata.path, err => {
        if (err) console.error(`Error removing file ${metadata.path}:`, err);
      });
      delete filesMetadata[fileId];
    }
  }
  fs.writeFileSync(filesMetadataPath, JSON.stringify(filesMetadata, null, 2));
}, 60000); // Run every minute

app.listen(port, () => {
  console.log(`Server is running on http://0.0.0.0:${port}`);
});
